/*
 * Copyright 2017 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.ispras.retrascope.engine.cfg;

import ru.ispras.retrascope.model.basis.Statement;
import ru.ispras.retrascope.model.basis.walker.ModelWalker;
import ru.ispras.retrascope.model.basis.walker.WalkerMode;
import ru.ispras.retrascope.model.cfg.CfgDfsWalker;

/**
 * Utility methods for control flow graph model transformation.
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public final class CfgSsaTransformer {

  private CfgSsaTransformer() {
    /* Empty. */
  }

  /**
   * Transforms th specified statement to a Static Single Assignment (SSA) form.
   *
   * @param statement The statement to be transformed.
   */
  public static void toSsa(final Statement statement) {

    final ModelWalker walker = new CfgDfsWalker(new CfgSsaVisitor(), WalkerMode.DFS);
    walker.visitNode(statement);
  }
}
