/*
 * Copyright 2014-2017 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.ispras.retrascope.engine.cgaa.transformer.efsm;

import ru.ispras.fortress.expression.NodeVariable;
import ru.ispras.retrascope.model.basis.Assignment;
import ru.ispras.retrascope.model.basis.Event;
import ru.ispras.retrascope.model.basis.ModuleProcess;
import ru.ispras.retrascope.model.basis.VariableType;
import ru.ispras.retrascope.model.cfg.CfgBlockStatement;

import java.util.LinkedHashSet;
import java.util.Set;

/**
 * Component that detects clock-like variables in the Clocked Guarded Atomic Actions (CGAA) model.
 *
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
final class CgaaClockDetector extends MapCfgVisitor {

  /* The clock-like variables for current process. */
  private Set<NodeVariable> clocks;

  @Override
  public void onProcessBegin(final ModuleProcess process) {

    this.clocks = new LinkedHashSet<>();

    for (final Event event : process.getEventList()) {

      final NodeVariable variable = event.getVariable();

      if (VariableType.isOneBitSignal(variable)) {
        this.clocks.add(variable);
      }
    }
  }

  @Override
  public void onProcessEnd(final ModuleProcess process) {

    this.getClockMap().put(process, new LinkedHashSet<>(this.clocks));
    this.clocks.clear();
  }

  @Override
  public void onBlockBegin(final CfgBlockStatement block) {

    for (final Assignment statement : block.getAssignments()) {

      for (final NodeVariable stmtUse : statement.getUses()) {
        this.clocks.remove(stmtUse);
      }

      for (final NodeVariable stmtDefine : statement.getDefines()) {
        this.clocks.remove(stmtDefine);
      }
    }
  }
}
