/*
 * Copyright 2015 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.engine.cgaa.transformer.hldd;

import ru.ispras.retrascope.basis.Engine;
import ru.ispras.retrascope.basis.Entity;
import ru.ispras.retrascope.basis.EntityType;
import ru.ispras.retrascope.model.basis.walker.ModelWalker;
import ru.ispras.retrascope.model.basis.walker.WalkerMode;
import ru.ispras.retrascope.model.cfg.CfgDfsWalker;
import ru.ispras.retrascope.model.cfg.CfgModel;
import ru.ispras.retrascope.model.cgaa.CgaaModel;
import ru.ispras.retrascope.model.hldd.HlddModel;
import ru.ispras.retrascope.model.hldd.HlddWalker;

import java.util.Map;

/**
 * This engine transforms the cgaa model into the HLDD model.
 *
 * @author <a href="mailto:lebedev@ispras.ru">Mikhail Lebedev</a>
 */
public class CgaaHlddTransformer extends Engine {

  /** The engine identifier. */
  public static final String ENGINE_ID = "cgaa-hldd-transformer";

  /**
   * Constructs a new engine.
   */
  public CgaaHlddTransformer() {
    super(ENGINE_ID);
    addInputType(CgaaModel.TYPE);
    setOutputType(HlddModel.TYPE);
  }

  @Override
  public Entity start(final Map<EntityType, Entity> inputs) {

    final CgaaModel cgaaModel = (CgaaModel) inputs.get(CgaaModel.TYPE);
    final CfgModel cfgModel = cgaaModel.getModel();
    final CgaaHlddVisitor visitor = new CgaaHlddVisitor();
    final ModelWalker walker = new CfgDfsWalker(visitor, WalkerMode.DFS);
    walker.start(cfgModel);

    final HlddModel model = visitor.getModel();

    final HlddOptimizationVisitor optimizer = new HlddOptimizationVisitor();
    final HlddWalker hlddWalker = new HlddWalker(optimizer);
    hlddWalker.start(model);

    return model;
  }

}
