/*
 * Copyright 2014-2016 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.ispras.retrascope.engine.efsm;

import ru.ispras.retrascope.basis.Engine;
import ru.ispras.retrascope.basis.Entity;
import ru.ispras.retrascope.basis.EntityType;
import ru.ispras.retrascope.model.basis.walker.ModelVisitor;
import ru.ispras.retrascope.model.basis.walker.ModelWalker;
import ru.ispras.retrascope.model.efsm.EfsmDfsWalker;
import ru.ispras.retrascope.model.efsm.EfsmModel;
import ru.ispras.retrascope.model.efsm.EfsmVisitor;

import java.util.Map;

/**
 * The abstract class for tools which deal with EFSMs.
 *
 * @author <a href="mailto:i.melnichenko@deltasolutions.ru">Igor Melnichenko</a>
 */
public abstract class EfsmEngine extends Engine {

  protected final ModelVisitor visitor;

  /**
   * Constructs an EFSM engine with the specified ID, visitor, and output type.
   *
   * @param id the ID
   * @param visitor the visitor
   * @param outputType the output type
   */
  protected EfsmEngine(
      final String id,
      final ModelVisitor visitor,
      final EntityType outputType) {
    super(id, EfsmModel.TYPE, outputType);

    this.visitor = visitor;
  }

  /**
   * Initialises this EFSM engine with the specified inputs.
   *
   * @param inputs a map containing mappings between entity types and corresponding entities
   */
  protected abstract void initialise(Map<EntityType, Entity> inputs);

  @Override
  public Entity start(final Map<EntityType, Entity> inputs) {
    this.initialise(inputs);

    final ModelWalker walker = new EfsmDfsWalker((EfsmVisitor)this.visitor);
    final EfsmModel model = (EfsmModel) inputs.get(EfsmModel.TYPE);
    walker.start(model);

    return this.getOutput();
  }

  /**
   * Returns an output entity of this engine.
   *
   * @return the output entity
   */
  protected abstract Entity getOutput();
}
