/*
 * Copyright 2015 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.ispras.retrascope.engine.efsm.extractor.assertion;

import ru.ispras.retrascope.basis.Entity;
import ru.ispras.retrascope.basis.EntityType;
import ru.ispras.retrascope.engine.efsm.EfsmEngine;
import ru.ispras.retrascope.result.assertion.AssertionEntity;

import java.util.Map;

/**
 * Engine that extracts transition assertions from EFSM models.
 * <p>
 * Transition assertion is a condition which determines suitable values of variables
 * for the transition to be fired. For example, when the transition starts from the
 * state which has a constraint "X" on state-like variables, and the transition has a guard "G",
 * then the transition assertion is "X AND G".
 * </p>
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public class EfsmTransitionAssertExtractor extends EfsmEngine {

  /**
   * The engine identifier.
   */
  public static final String ENGINE_ID = "efsm-transition-assertion-extractor";

  /**
   * Default constructor.
   */
  public EfsmTransitionAssertExtractor() {
    super(ENGINE_ID, new EfsmTransitionAssertVisitor(), AssertionEntity.TYPE);
  }

  @Override
  public void initialise(Map<EntityType, Entity> inputs) {
    /* Empty. */
  }

  @Override
  public Entity getOutput() {
    return ((EfsmTransitionAssertVisitor)this.visitor).getOutput();
  }
}
