/*
 * Copyright 2014-2015 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.engine.efsm.generator.test.fate;

import ru.ispras.fortress.expression.NodeVariable;
import ru.ispras.retrascope.model.efsm.EfsmTransition;

/**
 * The representation of variable dependency between EFSM transitions.
 *
 * @author <a href="mailto:i.melnichenko@deltasolutions.ru">Igor Melnichenko</a>
 */
final class VariableDependency {
  private final EfsmTransition source;
  private final EfsmTransition destination;
  private final NodeVariable variable;

  VariableDependency(EfsmTransition sourceTransition, EfsmTransition destinationTransition,
      NodeVariable variable) {
    this.source = sourceTransition;
    this.destination = destinationTransition;
    this.variable = variable;
  }

  /* Returns a variable which this dependency is related to. */
  NodeVariable getVariable() {
    return this.variable;
  }

  @Override
  public int hashCode() {
    final int prime = 31;
    int result = 1;
    result = prime * result + ((destination == null) ? 0 : destination.hashCode());
    result = prime * result + ((source == null) ? 0 : source.hashCode());
    result = prime * result + ((variable == null) ? 0 : variable.hashCode());
    return result;
  }

  @Override
  public boolean equals(Object obj) {
    if (this == obj) {
      return true;
    }
    if (obj == null) {
      return false;
    }
    if (!(obj instanceof VariableDependency)) {
      return false;
    }
    VariableDependency other = (VariableDependency) obj;
    if (destination == null) {
      if (other.destination != null) {
        return false;
      }
    } else if (!destination.equals(other.destination)) {
      return false;
    }
    if (source == null) {
      if (other.source != null) {
        return false;
      }
    } else if (!source.equals(other.source)) {
      return false;
    }
    if (variable == null) {
      if (other.variable != null) {
        return false;
      }
    } else if (!variable.equals(other.variable)) {
      return false;
    }
    return true;
  }
}
