/*
 * Copyright 2015-2016 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.ispras.retrascope.engine.hldd.printer.smv;

import org.apache.commons.cli.ParseException;
import ru.ispras.retrascope.basis.Arguments;
import ru.ispras.retrascope.basis.Engine;
import ru.ispras.retrascope.basis.Entity;
import ru.ispras.retrascope.basis.EntityType;
import ru.ispras.retrascope.basis.FileNames;
import ru.ispras.retrascope.basis.Parameter;
import ru.ispras.retrascope.basis.exception.RetrascopeRuntimeException;
import ru.ispras.retrascope.model.hldd.HlddModel;
import ru.ispras.retrascope.model.hldd.HlddModelUtils;
import ru.ispras.retrascope.model.hldd.HlddVisitor;
import ru.ispras.retrascope.model.hldd.HlddWalker;

import java.util.Map;

/**
 * The printer for high level decision diagram (HLDD) models to SMV text format.
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public class HlddSmvPrinter extends Engine {

  /** The engine ID. */
  public static final String ENGINE_ID = "hldd-smv-printer";

  /** The engine output entity type. */
  public static final EntityType OUT_TYPE = EntityType.get("smv");

  /** The SMV output file name command-line parameter. */
  public static final Parameter SMV_FILE_NAME = new Parameter("smv-file", true,
      "Output SMV file name");

  /**
   * Default constructor.
   */
  public HlddSmvPrinter() {
    super(ENGINE_ID);

    addInputType(HlddModel.TYPE);
    setOutputType(OUT_TYPE);

    addParameter(SMV_FILE_NAME);
  }

  @Override
  public Entity start(Map<EntityType, Entity> inputs) {

    final FileNames result = new FileNames(OUT_TYPE.getId());

    try {

      final HlddModel model = (HlddModel) inputs.get(HlddModel.TYPE);

      if (!HlddModelUtils.containsEmptyModules(model)) {

        final Arguments arguments =
            parseCommandLine(((Arguments) inputs.get(Arguments.TYPE)).getCommandLine());
        final String fileName = arguments.getValue(SMV_FILE_NAME, "model.smv");
        result.add(fileName);

        final HlddVisitor visitor = new HlddSmvVisitor(fileName);

        final HlddWalker walker = new HlddWalker(visitor);
        walker.start(model);
      }

    } catch (final ParseException e) {
      throw new RetrascopeRuntimeException(e);
    }

    return result;
  }
}
