/*
 * Copyright 2016 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.engine.smv.test.parser.parsers;

import java.util.regex.Matcher;

/**
 * This class is used to parse an SMV model file and extract information about specifications.
 * 
 * @author <a href="mailto:lebedev@ispras.ru">Mikhail Lebedev</a>
 */
public class SmvSpecificationParser extends LogParser {

  private int overallSpecs;
  
  /**
   * Constructs a new parser.
   */
  public SmvSpecificationParser() {
    super();
    overallSpecs = 0;
  }
  
  /**
   * Parses a file for specification statistics.
   * 
   * @param fileName - the specified file name.
   */
  public void parseSpecifications(final String fileName) {
    init();
    parse(fileName);
  }

  @Override
  public void parseLine(final String line) {
    final Matcher specMatcher = specification.matcher(line);
    if (specMatcher.find()) {
      overallSpecs++;
    }
  }
  
  /**
   * Returns the total number of specifications.
   * 
   * @return The total number of specifications.
   */
  public int getOverallSpecs() {
    return overallSpecs;
  }

  private void init() {
    overallSpecs = 0;
  }

}
