/*
 * Copyright 2016 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.ispras.retrascope.engine.test.printer.testbench.verilog;

import ru.ispras.fortress.data.DataType;
import ru.ispras.retrascope.engine.test.printer.testbench.PlaceHolder;
import ru.ispras.retrascope.engine.test.printer.testbench.TestbenchPrinterEngine;

import java.util.HashMap;
import java.util.Map;

/**
 * Test printer to Verilog testbench format.
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public class TestVerilogTestbenchPrinter extends TestbenchPrinterEngine {

  /**
   * The engine identifier.
   */
  public static final String ENGINE_ID = "test-verilog-testbench-printer";

  private static final Map<DataType, String> DATA_TYPES_MAP = new HashMap<>();

  /**
   * Creates a new {@code TestVhdlTestbenchPrinter} instance.
   *
   */
  public TestVerilogTestbenchPrinter() {
    super(ENGINE_ID);
  }

  static {
    DATA_TYPES_MAP.put(DataType.BIT_VECTOR(1), "");
    DATA_TYPES_MAP.put(DataType.INTEGER, "");
    DATA_TYPES_MAP.put(DataType.BOOLEAN, "");
    DATA_TYPES_MAP.put(DataType.REAL, "");
  }

  @Override
  public String getTestbenchExt() {
    return "v";
  }

  @Override
  public String getTemplateResource() {
    return "templates/verilog/testbench";
  }

  @Override
  public String getTstParserResource() {
    return null;
  }

  @Override
  public String getInputPortKind() {
    return null;
  }

  @Override
  public String getOutputPortKind() {
    return null;
  }

  @Override
  public String getBitVectorFmt() {
    return String.format(
        "[%s:%s]",
        PlaceHolder.UPPER_BOUND,
        PlaceHolder.LOWER_BOUND);
  }

  @Override
  public String getPortDeclarationFmt() {
    return null;
  }

  @Override
  public String getInputDeclarationFmt() {
    return getSignalDeclarationFormat("reg");
  }

  @Override
  public String getOutputDeclarationFmt() {
    return getSignalDeclarationFormat("wire");
  }

  @Override
  public String getBufferDeclarationFmt() {
    return null;
  }

  @Override
  public String getPortMapDeclarationFmt() {
    return  String.format(
        "%s.%s(%s)",
        PlaceHolder.TAB,
        PlaceHolder.PORT_NAME,
        PlaceHolder.SIGNAL_NAME);
  }

  @Override
  public String getSignalAssignmentFmt() {
    return null;
  }

  @Override
  public String getBufferAssignmentFmt() {
    return null;
  }

  @Override
  public Map<DataType, String> getDataTypeMap() {
    return DATA_TYPES_MAP;
  }

  @Override
  public Map<String, String> getConvertFunctionMap() {
    return null;
  }

  private static String getSignalDeclarationFormat(final String type) {
    return String.format(
        "%s%s %s %s;",
        PlaceHolder.TAB,
        type,
        PlaceHolder.SIGNAL_TYPE,
        PlaceHolder.SIGNAL_NAME);
  }
}
