/*
 * Copyright 2015-2016 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.ispras.retrascope.model.basis;

import java.util.List;

/**
 * Container of assignments.
 *
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public interface AssignmentContainer {

  /**
   * Adds assignment.
   *
   * @param stmt Assignment to be added.
   */
  void addAssignment(final Assignment stmt);

  /**
   * Appends assignments list to the last assignment of this object.
   *
   * @param statements Assignments to be added.
   */
  void addAssignments(final List<Assignment> statements);

  /**
   * Returns all assignments.
   *
   * @return Assignments.
   */
  List<Assignment> getAssignments();

  /**
   * Removes all assignments.
   */
  void removeAssignments();

  /**
   * Checks rather this object includes concurrent or blocking assignments.
   * @return {@code true} if this object contains concurrent assignments only,
   *         {@code false} if this object contains blocking assignments only.
   */
  boolean isConcurrent();
}
