/*
 * Copyright 2014-2017 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.model.basis.jaxb;

import ru.ispras.retrascope.model.basis.Event;
import ru.ispras.retrascope.model.basis.EventList;

import javax.xml.bind.annotation.adapters.XmlAdapter;

/**
 * The adapter class for JAXB marshalling/unmarshalling of {@link EventList} objects.
 *
 * <p>Performs conversion between {@link EventList} and {@link JaxbEventList} instances.</p>
 *
 * @author <a href="mailto:i.melnichenko@deltasolutions.ru">Igor Melnichenko</a>
 *
 * @see EventList
 * @see JaxbEventList
 */
public class JaxbEventListAdapter extends XmlAdapter<JaxbEventList, EventList> {

  @Override
  public JaxbEventList marshal(final EventList eventList) throws Exception {

    if (eventList == null) {
      return null;
    }

    final JaxbEventList jaxbEventList = new JaxbEventList();
    final JaxbEventAdapter jaxbEventAdapter = new JaxbEventAdapter();

    for (final Event event: eventList) {
      jaxbEventList.jaxbEvents.add(jaxbEventAdapter.marshal(event));
    }

    return jaxbEventList;
  }

  @Override
  public EventList unmarshal(final JaxbEventList jaxbEventList) throws Exception {
    if (jaxbEventList == null) {
      return null;
    }

    final EventList eventList = new EventList();
    final JaxbEventAdapter jaxbEventAdapter = new JaxbEventAdapter();

    for (final JaxbEvent jaxbEvent: jaxbEventList.jaxbEvents) {
      eventList.add(jaxbEventAdapter.unmarshal(jaxbEvent));
    }

    return eventList;
  }
}
