/*
 * Copyright 2014-2016 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.model.basis.memory;

import ru.ispras.fortress.expression.NodeValue;

import java.util.Map;

/**
 * The interface for a store of variables.
 *
 * @author <a href="mailto:i.melnichenko@deltasolutions.ru">Igor Melnichenko</a>
 */
interface Memory {
  /**
   * Sets a variable with the specified name to the specified value.
   *
   * @param name the name of the modified variable
   * @param value the value to be set
   */
  void setValue(final String name, final NodeValue value);

  /**
   * Sets variables with the specified names to the specified values.
   *
   * @param values a map containing mappings between names and new values of variables to be
   *        modified
   */
  void setValues(final Map<String, NodeValue> values);

  /**
   * Checks if the specified variable is stored in this memory.
   *
   * @param name a variable name
   *
   * @return {@code true} if this memory contains a value of the specified variable. Otherwise
   *         returns {@code false}
   */
  boolean containsVariable(final String name);

  /**
   * Returns a value of the specified variable.
   *
   * @param name a variable name
   *
   * @return the variable value
   *
   * @throws IllegalArgumentException if the specified variable is not found in this memory
   */
  NodeValue getValue(final String name);

  /**
   * Returns a values of all stored variables.
   *
   * @return a map containing mappings between names and values of the variables
   */
  Map<String, NodeValue> getValues();

  /**
   * Sets all variables of this memory to the same values as they had immediately after the memory
   * construction.
   */
  void reset();
}
