/*
 * Copyright 2016-2017 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.ispras.retrascope.model.basis.walker;

import ru.ispras.retrascope.model.basis.Statement;

import java.util.LinkedList;
import java.util.Queue;

/**
 * Breadth-first search model walker.
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public class BfsWalker extends ModelWalker {

  /* Queue for statements to be visited. */
  private final Queue<Statement> bfsStatements = new LinkedList<>();

  /**
   * Constructs walker object with the specified parameters.
   *
   * @param visitor visitor to be applied to control flow graph model
   * @throws IllegalArgumentException when any argument is {@code null}.
   */
  protected BfsWalker(final ModelVisitor visitor) {
    super(visitor, WalkerMode.BFS);
  }

  @Override
  protected void visitModelStatement(Statement statement) {
    /* Empty. */
  }

  @Override
  public void processChildren(final Statement statement) {

    for (int i = 0; i < statement.getStatementNum(); i++) {

      final Statement child = statement.getStatement(i);
      final String childId = child.getId();

      if (!getVisited().contains(childId)) {
        bfsStatements.add(child);
      }
    }
  }

  @Override
  public void goToNextStatement() {

    final Statement next = bfsStatements.poll();
    final String nextId = next.getId();
    visitNode(bfsStatements.poll());
    addVisited(nextId);
  }
}
