/*
 * Copyright 2015-2016 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.model.cfg;

import ru.ispras.retrascope.model.basis.Statement;
import ru.ispras.retrascope.util.LogMessage;

import java.util.Collection;

/**
 * Statement that does not contain any other statements.
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
abstract class CfgAtomicStatement extends CfgStatement {

  CfgAtomicStatement() {
    super();
  }

  CfgAtomicStatement(final CfgAtomicStatement statement) {
    super(statement);
  }

  /**
   * Returns the number of child statements.
   *
   * @return The number of child statements.
   */
  @Override
  public int getStatementNum() {
    return 0;
  }

  @Override
  public Statement getStatement(final int num) {
    return null;
  }

  @Override
  public void add(final Statement statement) {
    unsupported();
  }

  @Override
  public void remove(final Statement statement) {
    unsupported();
  }

  @Override
  public void addAll(final Collection<Statement> list) {
    unsupported();
  }

  private void unsupported() {
    throw new UnsupportedOperationException(
        LogMessage.makeMessage(LogMessage.NO_CHILD, getClass().getName()));
  }
}
