/*
 * Copyright 2015-2016 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.model.efsm;

import ru.ispras.fortress.util.InvariantChecks;
import ru.ispras.retrascope.basis.EntityType;
import ru.ispras.retrascope.model.basis.Model;

import java.util.Collection;

/**
 * The ordered container for EFSM modules.
 *
 * @author <a href="mailto:i.melnichenko@deltasolutions.ru">Igor Melnichenko</a>
 */
public class EfsmModel extends Model {

  /**
   * The entity type of all {@code EfsmModel} entities.
   */
  public static final EntityType TYPE = EntityType.get("efsm");

  /**
   * Creates an empty EFSM model.
   */
  public EfsmModel() {
    super(EfsmModel.TYPE);
  }

  /**
   * Creates an EFSM model containing the specified EFSM module.
   *
   * @param efsmModule the EFSM module to be placed into the EFSM model
   *
   * @throws IllegalArgumentException if {@code efsmModule} is {@code null}
   */
  public EfsmModel(EfsmModule efsmModule) {
    this();

    InvariantChecks.checkNotNull(efsmModule);
    addModule(efsmModule);
  }

  /**
   * Creates an EFSM model containing the specified EFSM modules.
   *
   * @param efsmModules the EFSM modules to be placed into the EFSM model
   *
   * @throws IllegalArgumentException if {@code efsmModules} is {@code null}
   */
  public EfsmModel(Collection<EfsmModule> efsmModules) {
    this();

    InvariantChecks.checkNotNull(efsmModules);
    addModules(efsmModules);
  }

  @Override
  public int hashCode() {
    final int prime = 31;
    final int result = super.hashCode();
    return prime * result;
  }

  @Override
  public boolean equals(Object object) {
    return this == object || !(!super.equals(object) || !(object instanceof EfsmModel));
  }
}
