/*
 * Copyright 2015-2016 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.model.hldd;

import ru.ispras.retrascope.model.basis.Module;
import ru.ispras.retrascope.model.basis.Statement;
import ru.ispras.retrascope.model.cfg.CfgModule;

/**
 * This class represents a module of a HLDD model. Contains a collection of HlddProcesses and a
 * corresponding CFG {@link CfgModule}.
 *
 * @author <a href="mailto:lebedev@ispras.ru">Mikhail Lebedev</a>
 */
public final class HlddModule extends Module {

  /**
   * Constructs a new module and sets the specified data.
   *
   * @param module - the specified CFG {@link CfgModule}.
   */
  public HlddModule(final Module module) {
    super(module.getName());
    declareVariables(module.getDeclarationMap());
  }

  /**
   * Constructs a new module and sets the specified data.
   *
   * @param module - the specified CFG {@link CfgModule}.
   */
  private HlddModule(final HlddModule module) {
    super(module);
  }

  @Override
  public String toString() {
    return "HlddModule [module=" + getName() + "]";
  }

  @Override
  public Statement deepCopy() {
    return new HlddModule(this);
  }
}
