/*
 * Copyright 2017 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.model.hldd;

import java.util.Collection;

/**
 * HLDD non-terminal node interface.
 *
 * @author <a href="mailto:lebedev@ispras.ru">Mikhail Lebedev</a>
 */
interface HlddNonTerminalNode {

  /**
   * Returns the children nodes.
   *
   * @return Collection of children.
   */
  Collection<? extends HlddNode> getChildren();

  /**
   * Adds the specified {@link HlddNode} to the children nodes.
   *
   * @param child - the specified child node.
   */
  void addChild(final HlddNode child);

  /**
   * Removes the specified {@link HlddNode} from the children nodes.
   *
   * @param child - the specified child node.
   */
  void removeChild(final HlddNode child);
}
