/*
 * Copyright 2015-2016 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.model.hldd;

import ru.ispras.retrascope.model.basis.walker.ModelVisitor;

/**
 * This class represents the {@link HlddModel} visitor interface.
 *
 * @author <a href="mailto:lebedev@ispras.ru">Mikhail Lebedev</a>
 */
public interface HlddVisitor extends ModelVisitor {

  /**
   * Starts visiting {@link Hldd}.
   *
   * @param hldd - the specified hldd.
   */
  void onHlddBegin(Hldd hldd);

  /**
   * Finishes visiting {@link Hldd}.
   *
   * @param hldd - the specified hldd.
   */
  void onHlddEnd(Hldd hldd);

  /**
   * Starts visiting the root node of a {@link Hldd}.
   *
   * @param rootNode - the specified {@link HlddNode}.
   */
  void onRootBegin(HlddNode rootNode);

  /**
   * Finishes visiting the root node of a {@link Hldd}.
   *
   * @param rootNode - the specified {@link HlddNode}.
   */
  void onRootEnd(HlddNode rootNode);

  /**
   * Starts visiting a condition node.
   *
   * @param node - the specified {@link HlddConditionNode}.
   */
  void onConditionBegin(HlddConditionNode node);

  /**
   * Finishes visiting a condition node.
   *
   * @param node - the specified {@link HlddConditionNode}.
   */
  void onConditionEnd(HlddConditionNode node);

  /**
   * Starts visiting a value node.
   *
   * @param node - the specified {@link HlddEdge}.
   */
  void onEdgeBegin(HlddEdge node);

  /**
   * Finishes visiting a value node.
   *
   * @param node - the specified {@link HlddEdge}.
   */
  void onEdgeEnd(HlddEdge node);

  /**
   * Visits an assignment node.
   *
   * @param node - the specified {@link HlddTerminalNode}.
   */
  void onAssignment(HlddTerminalNode node);

}
