/*
 * Copyright 2015-2017 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.ispras.retrascope.util;

/**
 * Utility class for logger messages.
 *
 * @see java.util.logging.Logger
 *
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public enum LogMessage {

  ALREADY_CONTAIN("Already contain"),
  EMPTY_COLLECTION("Empty collection"),
  CANT_CREATE_FILE("Can't create file"),
  CANT_FIND("Can't find"),
  CANT_PARSE("Can't parse"),
  CANT_READ("Can't read"),
  CANT_WRITE("Can't write"),
  CANT_REASSIGN("Unable to reassign with"),
  NO_CHILD("Has no children"),
  NO_SUCH("No such node"),
  UNSUPPORTED_TYPE("Unsupported type(s)"),
  WRONG_ARG("Wrong arg(s)"),
  WRONG_TYPE("Wrong type(s)"),
  ;

  private final String description;

  LogMessage(final String description) {
    this.description = description;
  }

  /**
   * Creates message string with the specified prefix and the specified data stored in object.
   * @param prefix Message prefix.
   * @param obj Message data.
   * @return A {@code String} message starting from the specified prefix and containing data
   *         stored in the specified object.
   */
  public static String makeMessage(final Enum<?> prefix, final Object obj)  {

    return String.format("%s%s.", prefix, obj);
  }

  @Override
  public String toString() {

    return String.format("%s: ", this.description);
  }
}
