/*
 * Copyright 2015-2016 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.ispras.retrascope.util;

import ru.ispras.fortress.expression.Node;
import ru.ispras.fortress.expression.NodeValue;
import ru.ispras.fortress.solver.SolverResult;

/**
 * Utility methods to work with solver.
 *
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public final class SolverUtils {

  /**
   * The "true" syntax node.
   */
  public static final NodeValue TRUE = NodeValue.newBoolean(true);

  /**
   * The "false" syntax node.
   */
  public static final NodeValue FALSE = NodeValue.newBoolean(false);

  private SolverUtils() {
    /* Empty. */
  }

  /**
   * Checks whether the specified expression is satisfiable.
   *
   * @param node Expression to be checked.
   * @return {@code true} if the expression is satisfiable or {@code false} otherwise.
   *
   */
  public static boolean isSat(final Node node) {

    return SatSolver.getInstance().isSat(node);
  }

  /**
   * Checks whether the specified logical expressions are compatible
   * {@code (getConjunction(nodes[0], ..., nodes[n-1]) is SAT)}.
   *
   * @param expressions Logical expressions to be checked.
   *
   * @return {@code true} if the specified expressions are compatible, {@code false} otherwise.
   */
  public static boolean areCompatible(final Node... expressions) {

    return SatSolver.getInstance().areCompatible(expressions);
  }

  /**
   * Checks whether the specified logical expressions are complete
   * {@code (!getComplement(nodes[0], ..., nodes[n-1]) is SAT)}.
   *
   * @param expressions Logical expressions to be checked.
   *
   * @return {@code true} if the specified expressions are complete, {@code false} otherwise.
   */
  public static boolean areComplete(final Node... expressions) {

    return SatSolver.getInstance().areComplete(expressions);
  }

  /**
   * Solves the specified expression.
   *
   * @param node Expression to be solved.
   *
   * @return Result of solving.
   */
  public static SolverResult solve(final Node node) {

    return SatSolver.getInstance().solve(node);
  }
}
