/*
 * Copyright 2015-2016 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.ispras.retrascope.engine.cgaa.transformer.efsm;

import org.junit.Test;
import ru.ispras.retrascope.HdlTestFiles;
import ru.ispras.retrascope.Retrascope;
import ru.ispras.retrascope.model.efsm.EfsmModel;
import ru.ispras.retrascope.parser.vhdl.VhdlParser;

import java.util.Map;

/**
 * Test that extracts EFSM model from b11 design with the initially specified state-like variables.
 *
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public class B11CgaaEfsmTransformerTestCase {

  private static final String D_HYPHEN = "--";
  private static final String ITC99_PROJECT = "itc99";
  private static final String B11 = "b11";
  private static final String B11_STATE_VARS = "STATO";

  @Test
  public void runTest() {
    final Map<String, Map<String, String>> files = HdlTestFiles.getVhdlProjects();

    final Map<String, String> project = files.get(ITC99_PROJECT);

    String b11path = null;
    for (final Map.Entry<String, String> entry : project.entrySet()) {
      if (entry.getValue().equals(B11)) {
        b11path = entry.getKey();
      }
    }

    if (b11path != null) {

      Retrascope.main(
          D_HYPHEN + Retrascope.TARGET_PARAM.getLongName(),
          EfsmModel.TYPE.getId(),
          D_HYPHEN + Retrascope.ENGINE_PARAM.getLongName(),
          CgaaEfsmTransformer.ENGINE_ID,
          b11path,
          D_HYPHEN + VhdlParser.TOP_LEVEL.getLongName(),
          B11,
          D_HYPHEN + CgaaEfsmTransformer.STATE_VARIABLES.getLongName(),
          B11_STATE_VARS);
    }
  }
}
