/*
 * Copyright 2014-2016 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.ispras.retrascope.engine.efsm.printer.graphml;

import org.apache.commons.io.FilenameUtils;
import org.junit.Test;
import ru.ispras.fortress.util.InvariantChecks;

import ru.ispras.retrascope.HdlTestFiles;
import ru.ispras.retrascope.HdlTestUtils;
import ru.ispras.retrascope.Retrascope;
import ru.ispras.retrascope.basis.Configuration;
import ru.ispras.retrascope.engine.cgaa.transformer.efsm.CgaaEfsmTransformer;

import java.util.LinkedHashMap;
import java.util.Map;

/**
 * Test for {@link ru.ispras.retrascope.engine.efsm.printer.graphml.EfsmGraphMlPrinter}
 * printer for {@link ru.ispras.retrascope.model.efsm.EfsmModel} extended finite
 * state machine model.
 *
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public class EfsmGraphMlPrinterModuleTestCase {

  @Test
  public void runTest() {

    /* Get VHDL projects. */
    final Map<String, Map<String, String>> vhdlFiles = HdlTestFiles.getVhdlProjects();

    /* Run Retrascope on each module of each VHDL project. */
    for (final Map.Entry<String, Map<String, String>> entry : vhdlFiles.entrySet()) {
      final Map<String, String> projectMap = entry.getValue();

      for (final Map.Entry<String, String> moduleEntry : projectMap.entrySet()) {

        final Map<String, String> onlyFile = new LinkedHashMap<>();
        onlyFile.put(moduleEntry.getKey(), moduleEntry.getValue());

        final String fileName = moduleEntry.getValue() + "-efsm.graphml";

        HdlTestUtils.runVhdl(
            onlyFile,
            EfsmGraphMlPrinter.OUT_TYPE.getId(),
            "--" + Retrascope.ENGINE_PARAM.getLongName(),
            EfsmGraphMlPrinter.ENGINE_ID,
            "--" + Configuration.PRINT_STYLE_PARAM.getLongName(),
            "VHDL",
            "--" + EfsmGraphMlPrinter.OUT_FILE_NAME.getLongName(),
            fileName,
            "--" + Retrascope.ENGINE_PARAM.getLongName(),
            CgaaEfsmTransformer.ENGINE_ID);
      }
    }

    /* Get Verilog projects. */
    final Map<String, Map<String, String>> verilogFiles = HdlTestFiles.getVerilogProjects();

    /* Run Retrascope on each module of each Verilog project. */
    for (final Map.Entry<String, Map<String, String>> entry : verilogFiles.entrySet()) {
      final Map<String, String> projectMap = entry.getValue();

      for (final Map.Entry<String, String> moduleEntry : projectMap.entrySet()) {
        final Map<String, String> onlyFile = new LinkedHashMap<>();
        final String fileName = moduleEntry.getKey();
        onlyFile.put(fileName, moduleEntry.getValue());

        InvariantChecks.checkNotNull(onlyFile.get(fileName));

        /* For Verilog modules there are no toplevels,
          so GraphML file name must be extracted from filename itself. */
        final String graphMlFileName = FilenameUtils.getBaseName(fileName);
        HdlTestUtils.runVerilog(
            onlyFile,
            EfsmGraphMlPrinter.OUT_TYPE.getId(),
            "--" + Retrascope.ENGINE_PARAM.getLongName(),
            EfsmGraphMlPrinter.ENGINE_ID,
            "--" + Configuration.PRINT_STYLE_PARAM.getLongName(),
            "VERILOG",
            "--" + EfsmGraphMlPrinter.OUT_FILE_NAME.getLongName(),
            graphMlFileName + "-efsm.graphml",
            "--" + Retrascope.ENGINE_PARAM.getLongName(),
            CgaaEfsmTransformer.ENGINE_ID);
      }
    }
  }
}
