/*
 * Copyright 2015-2016 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.engine.hldd.printer.smv.assertion;

import org.apache.commons.io.FilenameUtils;
import org.junit.Assert;
import org.junit.Test;
import ru.ispras.retrascope.HdlTestFiles;
import ru.ispras.retrascope.HdlTestUtils;
import ru.ispras.retrascope.Retrascope;
import ru.ispras.retrascope.engine.cgaa.transformer.efsm.CgaaEfsmTransformer;
import ru.ispras.retrascope.util.ReadWriteUtils;

import java.io.File;
import java.io.PrintWriter;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Map.Entry;

/**
 * This class represents the test case for {@link HlddAssertSmvPrinter} class.
 *
 * @author <a href="mailto:lebedev@ispras.ru">Mikhail Lebedev</a>
 */
public final class HlddAssertSmvPrinterTestCase {

  @Test
  public void runTest() {

    final Map<String, Map<String, String>> files = HdlTestFiles.getProjects();

    for (Entry<String, Map<String, String>> project : files.entrySet()) {
      for (Entry<String, String> file : project.getValue().entrySet()) {

        final String filePath = file.getKey();

        final Map<String, String> onlyFile = new LinkedHashMap<>();
        onlyFile.put(filePath, file.getValue());
        final String designName = FilenameUtils.getBaseName(filePath);
        final String smvFileName = designName + ".smv";

        /* If SMV file exists, clear it. */
        final File smvFile = new File(smvFileName);
        if (smvFile.exists()) {
          final PrintWriter writer = ReadWriteUtils.newPrintWriter(smvFile);
          writer.write("");
          writer.close();
        }

        if ("b07".equals(designName) || "b11".equals(designName) || "b08".equals(designName)) {
          HdlTestUtils.runVhdl(
              onlyFile,
              HlddAssertSmvPrinter.OUT_TYPE.getId(),
              "--" + Retrascope.ENGINE_PARAM.getLongName(),
              HlddAssertSmvPrinter.ENGINE_ID,
              "--" + HlddAssertSmvPrinter.SMV_FILE_NAME.getLongName(),
              smvFileName,
              "--" + CgaaEfsmTransformer.STATE_VARIABLES.getLongName(),
              "STATO");
        } else {
          HdlTestUtils.runHdl(
              onlyFile,
              HlddAssertSmvPrinter.OUT_TYPE.getId(),
              "--" + Retrascope.ENGINE_PARAM.getLongName(),
              HlddAssertSmvPrinter.ENGINE_ID,
              "--" + HlddAssertSmvPrinter.SMV_FILE_NAME.getLongName(),
              smvFileName);
        }

        if (!"empty.smv".equals(smvFileName) && !smvFileName.equals("mips_16_defs.smv")) {
          Assert.assertTrue(exists(smvFileName));
        }
      }
    }
  }

  private boolean exists(final String name) {
    final File file = new File(name);
    return file.exists();
  }
}
