/*
 * Copyright 2016 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.ispras.retrascope.engine.test.printer.testbench.verilog;

import org.junit.Test;
import ru.ispras.retrascope.HdlTestUtils;
import ru.ispras.retrascope.Retrascope;
import ru.ispras.retrascope.engine.efsm.generator.test.EfsmTestGenerator;
import ru.ispras.retrascope.engine.test.printer.testbench.TestbenchPrinterEngine;
import ru.ispras.retrascope.result.testbench.TestbenchModel;

import java.io.File;
import java.io.IOException;
import java.nio.file.Paths;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;

/**
 * Test for Verilog testbench printer.
 *
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public class TestVerilogTestbenchPrinterVerilogTestCase {

  private static final String HYP = "--";

  /*
   * Designs which cause failures at the moment.
   */
  private static final String[] IGNORED_PATHS = {
    Paths.get("adder", "adder4.v").toString(),
    Paths.get("adder", "adder4_testbench.v").toString(),
    Paths.get("blocks", "blocks.v").toString(),
    Paths.get("bv_non_zero_idx", "bv_non_zero_idx.v").toString(),
    Paths.get("fifo", "fifo.v").toString(),
    Paths.get("fifo", "fifo_testbench.v").toString(),
    Paths.get("incompat_cond", "incompat_cond.v").toString(),
    Paths.get("mux", "mux4_1.v").toString(),
    Paths.get("ram", "ram.v").toString(),
    Paths.get("ram", "ram_testbench.v").toString(),
  };

  private static final Set<String> IGNORED = new HashSet<>(Arrays.asList(IGNORED_PATHS));

  @Test
  public void generate() throws IOException {
    HdlTestUtils.runVerilog(
        TestbenchModel.TYPE.toString(),
        IGNORED,
        HYP + Retrascope.ENGINE_PARAM.getLongName(),
        EfsmTestGenerator.ENGINE_ID + File.pathSeparator + TestVerilogTestbenchPrinter.ENGINE_ID,
        HYP + TestbenchPrinterEngine.OVERWRITE_EXISTING.getLongName(),
        HYP + EfsmTestGenerator.LOOP_LIMIT_PARAMETER.getLongName(), "25"//25 is for b11
            /*, "--" + Retrascope.LOG_LEVEL.getLongName(), "DEBUG"*/);
  }
}
