/*
 * Copyright 2014-2016 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.model.efsm;

import org.junit.Assert;
import org.junit.Test;

import ru.ispras.retrascope.SolverUtilTest;
import ru.ispras.retrascope.model.basis.Module;
import ru.ispras.retrascope.model.efsm.examples.B04;
import ru.ispras.retrascope.util.Log;

import java.util.LinkedList;
import java.util.List;

/**
 * @author <a href="mailto:lebedev@ispras.ru">Mikhail Lebedev</a>
 */
public class ExampleEfsmProductTestCase extends SolverUtilTest {

  @Test
  public void testProduct() {
    final EfsmModel model1 = B04.EFSM_MODEL;
    final EfsmModel model2 = B04.EFSM_MODEL;
    final List<Module> model1Modules = new LinkedList<>(model1.getModules());
    final List<Module> model2Modules = new LinkedList<>(model2.getModules());

    final EfsmModule module1 = (EfsmModule) model1Modules.get(0);
    final EfsmModule module2 = (EfsmModule) model2Modules.get(0);

    final Efsm b04 = module1.getNonInitialEfsms().iterator().next();
    final Efsm simpleEfsm = module2.getNonInitialEfsms().iterator().next();

    Log.printInfo(b04.getName());
    Log.printInfo("States: " + b04.getStateNum());
    Log.printInfo(b04.getStates());
    Log.printInfo("Transitions: " + b04.getTransitionNum());
    Log.printInfo(b04.getTransitions());
    Log.printInfo("");

    Log.printInfo(simpleEfsm.getName());
    Log.printInfo("States: " + simpleEfsm.getStateNum());
    Log.printInfo(simpleEfsm.getStates());
    Log.printInfo("Transitions: " + simpleEfsm.getTransitionNum());
    Log.printInfo(simpleEfsm.getTransitions());
    Log.printInfo("");

    final Efsm product = EfsmUtils.product(b04, simpleEfsm);

    Assert.assertNotNull(product);
    Assert.assertEquals(product.getStateNum(), b04.getStateNum() * simpleEfsm.getStateNum());

    Log.printInfo(product.getName());
    Log.printInfo("States: " + product.getStateNum());
    Log.printInfo(product.getStates());
    Log.printInfo("Transitions: " + product.getTransitionNum());
    for (final EfsmTransition item : product.getTransitions()) {
      Log.printInfo(item);
    }
    Log.printInfo("Initial state: " + product.getInitialState().toString());
    Log.printInfo("");

    final Efsm product2 = EfsmUtils.product(simpleEfsm, b04);

    Assert.assertNotNull(product2);
    Assert.assertEquals(product2.getStateNum(), simpleEfsm.getStateNum() * b04.getStateNum());

    Log.printInfo(product2.getName());
    Log.printInfo("States: " + product2.getStateNum());
    Log.printInfo("Transitions: " + product2.getTransitionNum());
    Log.printInfo("Initial state: " + product2.getInitialState());
    Log.printInfo("");

    final Efsm product3 = EfsmUtils.product(simpleEfsm, simpleEfsm, simpleEfsm);

    Assert.assertNotNull(product3);
    Assert.assertEquals(product3.getStateNum(),
        simpleEfsm.getStateNum() * simpleEfsm.getStateNum() * simpleEfsm.getStateNum());

    Log.printInfo(product3.getName());
    Log.printInfo("States: " + product3.getStateNum());
    Log.printInfo("Transitions: " + product3.getTransitionNum());
    Log.printInfo("Initial state: " + product3.getInitialState());
    Log.printInfo("");
  }
}
