/*
 * Copyright 2015-2016 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.ispras.retrascope.result.assertion;

import org.junit.Assert;
import org.junit.Test;
import ru.ispras.fortress.expression.Node;
import ru.ispras.fortress.expression.NodeValue;
import ru.ispras.fortress.expression.NodeVariable;
import ru.ispras.retrascope.basis.exception.RetrascopeRuntimeException;
import ru.ispras.retrascope.model.basis.Assignment;
import ru.ispras.retrascope.model.efsm.Action;
import ru.ispras.retrascope.model.efsm.GuardedAction;
import ru.ispras.retrascope.util.NodeUtils;
import ru.ispras.retrascope.util.SolverUtils;

import java.util.Collections;
import java.util.List;

/**
 * Test for assertion result's inner representation.
 *
 * @see AssertionEntity
 * @see AssertionContainer
 * @see AssertionGroup
 *
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public class AssertionEntityTestCase {

  @Test
  public void runTest() {

    /* Create top-level entity. */
    final AssertionEntity entity = new AssertionEntity();

    /* Create containers. */
    final AssertionContainer container1 = new AssertionContainer("1");
    entity.addContainer(container1);

    final AssertionContainer container2 = new AssertionContainer("2");
    entity.addContainer(container2);

    final AssertionContainer container3 = new AssertionContainer("3", container2);
    container2.addContainer(container3);

    /* Create groups. */
    final Node initState = SolverUtils.TRUE;
    final GuardedAction rstGuardedAction =
        new GuardedAction(
            initState,
            new Action(new Assignment(NodeVariable.newInteger("x"), NodeValue.newInteger(0))));

    final List<GuardedAction> rstGuardedActions = Collections.singletonList(rstGuardedAction);
    final AssertionGroup group1 = new AssertionGroup("1", container1, initState, rstGuardedActions);
    container1.addGroup(group1);
    Assert.assertTrue(container1.getGroupMap().size() == 1);

    final AssertionGroup group2 = new AssertionGroup("2", container2, initState, rstGuardedActions);
    container2.addGroup(group2);

    final AssertionGroup group3 = new AssertionGroup("3", container3, initState, rstGuardedActions);
    container3.addGroup(group3);

    /* Variables. */
    final NodeVariable varOne = NodeVariable.newInteger("one");
    final NodeVariable varTwo = NodeVariable.newInteger("two");
    final NodeVariable varThree = NodeVariable.newInteger("three");
    final NodeVariable varFour = NodeVariable.newInteger("four");

    /* Declare variables. */
    container1.declareVariable(varOne);
    container2.declareVariable(varTwo);
    container3.declareVariable(varThree);

    /* Assertions. */
    final Node equationX = NodeUtils.getEquation(varOne, NodeValue.newInteger(0));
    final Node equationY = NodeUtils.getEquation(varTwo, NodeValue.newInteger(0));
    final Node equationZ = NodeUtils.getEquation(varThree, NodeValue.newInteger(0));
    final Node equationW = NodeUtils.getEquation(varFour, NodeValue.newInteger(0));

    /* Try to add assertions. */
    group1.addAssertion(equationX);
    group2.addAssertion(equationY);
    group3.addAssertion(equationZ);

    /* Try to add assertion that uses undeclared variable. */
    try {
      group3.addAssertion(equationW);
    } catch (RetrascopeRuntimeException e) {
      return;
    }
    Assert.fail();
  }
}
