/*
 * Copyright 2015-2016 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.ispras.retrascope.util;

import org.junit.Assert;
import org.junit.Test;
import ru.ispras.fortress.expression.Node;
import ru.ispras.fortress.expression.NodeBinding;
import ru.ispras.fortress.expression.NodeOperation;
import ru.ispras.fortress.expression.NodeValue;
import ru.ispras.fortress.expression.NodeVariable;
import ru.ispras.fortress.expression.StandardOperation;

import java.util.LinkedList;
import java.util.List;

/**
 * Test for {@link TransformerUtils} class.
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public final class TransformerUtilsTestCase {

  private static final Node X_VAR = NodeVariable.newInteger("x");
  private static final Node Y_VAR = NodeVariable.newInteger("y");
  private static final Node Z_VAR = NodeVariable.newInteger("z");
  private static final Node V_VAR = NodeVariable.newInteger("v");

  private static final NodeValue ONE = NodeValue.newInteger(1);
  private static final NodeValue TWO = NodeValue.newInteger(2);
  private static final NodeValue THREE = NodeValue.newInteger(3);
  private static final NodeValue FOUR = NodeValue.newInteger(4);
  private static final NodeValue FIVE = NodeValue.newInteger(5);
  private static final NodeValue SIX = NodeValue.newInteger(6);

  /**
   * Default constructor.
   */
  public TransformerUtilsTestCase() {
    /* Empty. */
  }

  @Test
  public void runTest() {

    runMoreThanTwoOpEqTest();
    runAddSubModuleTest();
    runAddSubTest();
    runSubstituteBindingTest();
  }

  private void runSubstituteBindingTest() {

    final Node equation = NodeUtils.getEquation(X_VAR, Y_VAR);
    final List<NodeBinding.BoundVariable> bindings = new LinkedList<>();
    bindings.add(NodeBinding.bindVariable((NodeVariable) X_VAR, Z_VAR));
    bindings.add(NodeBinding.bindVariable((NodeVariable) Y_VAR, V_VAR));

    final Node applied = TransformerUtils.substituteBindings(equation, bindings);
    Assert.assertEquals(applied, NodeUtils.getEquation(Z_VAR, V_VAR));
  }

  private void runAddSubTest() {

    final Node expression =
        Node.AND(
            NodeUtils.getEquation(
                new NodeOperation(StandardOperation.ADD, X_VAR, ONE),
                FOUR),
            NodeUtils.getEquation(
                new NodeOperation(StandardOperation.SUB, X_VAR, TWO),
                THREE));

    final Node transformed = TransformerUtils.standardize(expression);

    final Node transformResult =
        Node.AND(
            NodeUtils.getEquation(X_VAR, THREE),
            NodeUtils.getEquation(X_VAR, FIVE));
    Assert.assertEquals(transformed, transformResult);
  }

  private void runAddSubModuleTest() {

    final Node expression =
        Node.AND(
            NodeUtils.getEquation(
                new NodeOperation(
                    StandardOperation.MOD,
                    new NodeOperation(StandardOperation.ADD, X_VAR, ONE),
                  FIVE),
              FOUR),
            NodeUtils.getEquation(
                new NodeOperation(
                    StandardOperation.MOD,
                    new NodeOperation(StandardOperation.SUB, X_VAR, TWO),
                  SIX),
              THREE));

    final Node transformed = TransformerUtils.standardize(expression);

    final Node transformResult =
        Node.AND(
            NodeUtils.getEquation(X_VAR, THREE),
            NodeUtils.getEquation(X_VAR, FIVE));
    Assert.assertEquals(transformed, transformResult);
  }

  private void runMoreThanTwoOpEqTest() {
    final Node moreThanTwoOp = NodeUtils.getEquation(X_VAR, Y_VAR, Z_VAR);

    final Node transformResult =
        Node.AND(NodeUtils.getEquation(Y_VAR, X_VAR), NodeUtils.getEquation(Z_VAR, X_VAR));

    final Node transformed = TransformerUtils.standardize(moreThanTwoOp);

    Assert.assertEquals(transformResult, transformed);
  }
}
